package main

import (
	"crypto/ecdh"
	"flag"
	"fmt"
	"os"

	"drat/internal/drat"
	"drat/internal/mu"
)

const usage = `Usage: alice [options] BOB_HOST

The "Alice" (initial sender) party of a double-ratchet communication.

positional arguments:
  BOB_HOST
    The a "host:port" string for Bob.  Host can be either an IP address or a
    hostname.
    
options:
  -secret SECRET_KEY
    The file that contains the 32-byte shared secret that serves as the
    initial root key.

    Default: secret.key

  -bob-pub BOB_PUBLIC_KEY
    A file with Bob's raw public X25519 key (that is, the file is the raw key
    bytes, rather than a DER or PEM-encoding of these bytes).

    Default: bob-pub.key

  -help
    Display this usage statement and exit.

examples:
$ ./alice -secret assets/secret.key -bob-pub assets/bob-pub.key 127.0.0.1:12345
`

type Options struct {
	// positional
	hostport string
	//options
	bobPubKeyFile string
	bobPubKey     *ecdh.PublicKey // derived
	secretKeyFile string
	secretKey     []byte // derived
}

func printUsage() {
	fmt.Fprintf(os.Stderr, "%s", usage)
}

func parseOptions() *Options {
	var err error
	options := Options{}

	flag.Usage = printUsage
	flag.StringVar(&options.bobPubKeyFile, "bob-pub", "bob-pub.key", "")
	flag.StringVar(&options.secretKeyFile, "secret", "secret.key", "")
	flag.Parse()

	if flag.NArg() != 1 {
		mu.Die("expected one positional argument but got %d", flag.NArg())
	}

	options.hostport = flag.Arg(0)

	options.secretKey, err = drat.LoadSecretKey(options.secretKeyFile)
	if err != nil {
		mu.Die("error loading secret key from %s: %v", options.secretKeyFile, err)
	}

	options.bobPubKey, err = drat.LoadPubDH(options.bobPubKeyFile)
	if err != nil {
		mu.Die("error loading Bob's public key from %s: %v", options.bobPubKeyFile, err)
	}

	return &options
}

func main() {
	options := parseOptions()

	// TODO: delete the mu.UNUSED line and implement the rest of the program
	mu.UNUSED(options)
}
